# -*- coding: utf-8 -*-
__all__ = ["ConstrainedImageField"]

import os

from django import forms
from django.conf import settings
from django.db import models
from django.template.defaultfilters import filesizeformat
from django.utils.translation import ugettext_lazy as _


class ConstrainedImageField(models.ImageField):
    """
    An ImageField with additional constraints. Namely, the file size and type can be restricted. If
    using the types, the magic library is required. Setting neither a file size nor type behaves
    like a regular FileField.

    Parameters
    ----------
    content_types : list of str
        List containing allowed content_types. Example: ['application/pdf', 'image/jpeg']
    [min|max]_upload_size : int
        Maximum file size allowed for upload, in bytes
            1 MB - 1048576 B - 1024**2 B - 2**20 B
            2.5 MB - 2621440 B
            5 MB - 5242880 B
            10 MB - 10485760 B
            20 MB - 20971520 B
            33 MiB - 2**25 B
            50 MB - 5242880 B
            100 MB 104857600 B
            250 MB - 214958080 B
            500 MB - 429916160 B
            1 GiB - 1024 MiB - 2**30 B
    [min|max]_upload_[heigth|width] : int
    js_checker : bool
        Add a javascript file size checker to the form field
    mime_lookup_length : int

    See Also
    --------
    Based on https://github.com/kaleidos/django-validated-file/blob/master/validatedfile/fields.py
    With inspiration from http://stackoverflow.com/a/9016664

    """

    description = _("An image file field with constraints on size and/or type")

    _constraint_prefix = "upload_"
    _constrained_fields = ["size", "height", "width"]

    @property
    def _constraints(self):
        return {
            field: self._constraint_prefix + field for field in self._constrained_fields
        }

    def __init__(self, *args, **kwargs):
        for attribute in self._constraints.values():
            setattr(self, attribute, {})
            for boundary in ["min", "max"]:
                value = kwargs.pop(boundary + "_" + attribute, 0)
                assert isinstance(value, int) and value >= 0
                getattr(self, attribute)[boundary] = value
        self.content_types = kwargs.pop("content_types", [])
        self.mime_lookup_length = kwargs.pop("mime_lookup_length", 4096)
        assert isinstance(self.mime_lookup_length, int) and self.mime_lookup_length >= 0
        self.js_checker = kwargs.pop("js_checker", False)

        super(ConstrainedImageField, self).__init__(*args, **kwargs)

    def clean(self, *args, **kwargs):
        data = super(ConstrainedImageField, self).clean(*args, **kwargs)
        errors = []

        for field in self._constrained_fields:
            attribute = getattr(self, self._constraints[field])
            below = attribute["min"] and getattr(data, field) < attribute["min"]
            above = attribute["max"] and getattr(data, field) > attribute["max"]
            if below or above:
                # Ensure no one bypasses the js checker
                errors.append(
                    _(
                        "File %(field)s "
                        + ("below" if below else "exceeds")
                        + " limit: %(current_size)s. Limit is %(limit)s."
                    )
                    % {
                        "field": field,
                        "limit": filesizeformat(attribute["min" if below else "max"])
                        if field == "size"
                        else attribute["min" if below else "max"],
                        "current_size": filesizeformat(data.size)
                        if field == "size"
                        else data.size,
                    }
                )

        if self.content_types:
            import magic

            file = data.file
            uploaded_content_type = getattr(file, "content_type", "")

            # magic_file_path used only for Windows.
            magic_file_path = getattr(settings, "MAGIC_FILE_PATH", None)
            if magic_file_path and os.name == "nt":
                mg = magic.Magic(mime=True, magic_file=magic_file_path)
            else:
                mg = magic.Magic(mime=True)
            content_type_magic = mg.from_buffer(file.read(self.mime_lookup_length))
            file.seek(0)

            # Prefer mime-type from magic over mime-type from http header
            if uploaded_content_type != content_type_magic:
                uploaded_content_type = content_type_magic

            if uploaded_content_type not in self.content_types:
                errors.append(
                    _("Unsupported file type: %(type)s. Allowed types are %(allowed)s.")
                    % {"type": content_type_magic, "allowed": self.content_types}
                )
        if errors:
            raise forms.ValidationError(errors)

        return data

    def formfield(self, **kwargs):
        """
        Usual Form for a django.models.FileField with optional javascript file size
        checker. Can thus be customized as any other Form for a django.models.FileField.

        Parameters
        ----------
        kwargs
            will be passed to super().formfield()

        Returns
        -------
        django.forms.FileField

        """
        formfield = super(ConstrainedImageField, self).formfield(**kwargs)
        if self.js_checker:
            formfield.widget.attrs.update(
                {
                    "onchange": "validateFileSize(this, %d, %d);"
                    % (
                        getattr(self, self._constraints["size"])["min"],
                        getattr(self, self._constraints["size"])["max"],
                    )
                }
            )
        return formfield

    def deconstruct(self):
        name, path, args, kwargs = super(ConstrainedImageField, self).deconstruct()
        for attribute in self._constraints.values():
            for boundary in ["min", "max"]:
                value = getattr(self, attribute)[boundary]
                if value:
                    kwargs[boundary + "_" + attribute] = value
        if self.content_types:
            kwargs["content_types"] = self.content_types
        if self.mime_lookup_length:
            kwargs["mime_lookup_length"] = self.mime_lookup_length
        if self.js_checker:
            kwargs["js_checker"] = self.js_checker
        return name, path, args, kwargs

    def __str__(self):
        if hasattr(self, "model"):
            return super(ConstrainedImageField).__str__()
        else:
            return self.__class__.__name__



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__license__ = "BSD-3-Clause"
__author__ = "Marc Bourqui"
__version__ = "4.0.0"
__version_info__ = tuple(
    [
        int(num) if num.isdigit() else num
        for num in __version__.replace("-", ".", 1).split(".")
    ]
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys

import django
from django.test.runner import DiscoverRunner

os.environ["DJANGO_SETTINGS_MODULE"] = "constrainedfilefield.tests.settings"

if __name__ == "__main__":
    # https://docs.djangoproject.com/en/1.8/topics/settings/#calling-django-setup-is-required-for-standalone-django-usage

    django.setup()

    sys.exit(
        DiscoverRunner(verbosity=1).run_tests(
            [
                "constrainedfilefield.tests.tests.backend",
                "constrainedfilefield.tests.tests.frontend",
            ]
        )
    )



from django.db import models

from constrainedfilefield.fields import ConstrainedFileField, ConstrainedImageField


class TestModel(models.Model):
    the_file = ConstrainedFileField(
        null=True,
        blank=True,
        upload_to="testfile",
        content_types=["image/png"],
        max_upload_size=10240,
    )


class TestDocModel(models.Model):
    the_file = ConstrainedFileField(
        null=False,
        blank=False,
        upload_to="testfile",
        content_types=[
            "application/msword",  # .doc
            "application/vnd.openxmlformats-officedocument.wordprocessingml.document",  # .docx
            "application/vnd.oasis.opendocument.text",  # .odt
        ],
        max_upload_size=10240,
    )


class TestImageModel(models.Model):
    the_image = ConstrainedImageField(null=True, blank=True, upload_to="testfile",)
    the_image_small = ConstrainedImageField(
        null=True,
        blank=True,
        upload_to="testfile",
        content_types=["image/png"],
        min_upload_size=1024,
        max_upload_size=10240,
        min_upload_height=100,
        max_upload_height=500,
        min_upload_width=100,
        max_upload_width=500,
    )
    the_image_large = ConstrainedImageField(
        null=True,
        blank=True,
        upload_to="testfile",
        content_types=["image/png"],
        min_upload_size=10240,
        max_upload_size=20480,
        min_upload_height=3000,
        max_upload_height=5000,
        min_upload_width=3000,
        max_upload_width=5000,
    )


class TestModelJs(models.Model):
    the_file = ConstrainedFileField(
        null=True,
        blank=True,
        upload_to="testfile",
        content_types=["image/png"],
        max_upload_size=10240,
        js_checker=True,
    )
    the_image = ConstrainedImageField(
        null=True,
        blank=True,
        upload_to="testfile",
        content_types=["image/png"],
        min_upload_size=1024,
        max_upload_size=10240,
        js_checker=True,
    )


class TestModelNoValidate(models.Model):
    the_file = ConstrainedFileField(null=True, blank=True, upload_to="testfile")
    the_image = ConstrainedImageField(null=True, blank=True, upload_to="testfile")


class TestContainer(models.Model):
    name = models.CharField(max_length=100)


class TestElement(models.Model):
    container = models.ForeignKey(
        TestContainer, on_delete=models.CASCADE, related_name="test_elements"
    )
    the_file = ConstrainedFileField(
        null=True,
        blank=True,
        upload_to="testfile",
        content_types=["image/png", "image/jpeg"],
    )



# -*- coding: utf-8 -*-
import os.path

from django.conf import settings
from django.core.files import File
from django.core.files.uploadedfile import SimpleUploadedFile
from django.test import TestCase

from constrainedfilefield.tests.forms import (
    TestModelForm,
    TestDocModelForm,
    TestImageModelForm,
    TestModelFormJs,
    TestModelNoValidateForm,
    TestElementForm,
    TestNoModelForm,
    TestNoModelJsForm,
)
from constrainedfilefield.tests.models import TestModel, TestImageModel, TestContainer


class ConstrainedFileFieldTest(TestCase):
    SAMPLE_FILES_PATH = os.path.join(settings.BASE_DIR, "sample_files")

    def test_create_empty_instance(self):
        TestModel.objects.create()

    def test_create_instance_with_file(self):
        instance = TestModel.objects.create(
            the_file=File(self._get_sample_file("image2k.png"), "the_file.png")
        )

        self._check_file_url(instance.the_file, "the_file.png")

        instance.the_file.delete()
        instance.delete()

    def test_create_instance_with_images(self):
        instance = TestImageModel.objects.create(
            the_image=File(self._get_sample_file("image2k.png"), "the_image.png"),
            the_image_small=File(
                self._get_sample_file("image2k.png"), "the_image_small.png"
            ),
            the_image_large=File(
                self._get_sample_file("image15k.png"), "the_image_large.png"
            ),
        )

        self._check_file_url(instance.the_image, "the_image.png")
        self._check_file_url(instance.the_image_small, "the_image_small.png")
        self._check_file_url(instance.the_image_large, "the_image_large.png")

        instance.the_image.delete()
        instance.the_image_small.delete()
        instance.the_image_large.delete()
        instance.delete()

    def test_form_ok(self):
        form = self._create_bound_test_model_form(
            form_class=TestModelForm,
            orig_filename="image2k.png",
            dest_filename="the_file.png",
            content_type="image/png",
        )
        self.assertTrue(form.is_valid())
        instance = form.save()

        self._check_file_url(instance.the_file, "the_file.png")

        instance.the_file.delete()
        instance.delete()

    def test_form_doc_ok(self):
        # ODT
        form = self._create_bound_test_model_form(
            form_class=TestDocModelForm,
            orig_filename="document15k.odt",
            dest_filename="the_file.odt",
            content_type="application/vnd.oasis.opendocument.text",
        )
        self.assertTrue(form.is_valid())
        instance = form.save()
        self._check_file_url(instance.the_file, "the_file.odt")
        instance.the_file.delete()
        instance.delete()

        # DOC
        form = self._create_bound_test_model_form(
            form_class=TestDocModelForm,
            orig_filename="document15k.doc",
            dest_filename="the_file.doc",
            content_type="application/msword",
        )
        self.assertTrue(form.is_valid())
        instance = form.save()
        self._check_file_url(instance.the_file, "the_file.doc")
        instance.the_file.delete()
        instance.delete()

        # DOCX
        # Requires a native .docx file
        # form = self._create_bound_test_model_form(form_class=TestModelDocForm,
        #                                           orig_filename='document15k.docx',
        #                                           dest_filename='the_file.docx',
        #                                           content_type='application/vnd.openxmlformats-officedocument.wordprocessingml.document')
        # self.assertTrue(form.is_valid())
        # instance = form.save()
        # self._check_file_url(instance.the_file, 'the_file.docx')
        # instance.the_file.delete()
        # instance.delete()

    def test_form_js_ok(self):
        form = self._create_bound_test_model_form(
            form_class=TestModelFormJs,
            orig_filename="image2k.png",
            dest_filename="the_file.png",
            content_type="image/png",
        )
        self.assertTrue(form.is_valid())
        instance = form.save()

        self._check_file_url(instance.the_file, "the_file.png")

        instance.the_file.delete()
        instance.delete()

    def test_form_invalid_size(self):
        form = self._create_bound_test_model_form(
            form_class=TestModelForm,
            orig_filename="image15k.png",
            dest_filename="the_file.png",
            content_type="image/png",
        )
        self.assertFalse(form.is_valid())
        self.assertEqual(len(form.errors), 1)
        self.assertEqual(len(form.errors["the_file"]), 1)
        self.assertEqual(
            form.errors["the_file"][0],
            u"File size exceeds limit: 14.2 KB. Limit is 10.0 KB.",
        )

    def test_form_valid_image_size(self):
        files = {
            "the_image": self._create_simple_uploaded_file(
                orig_filename="image2k.png",
                dest_filename="the_image.png",
                content_type="image/png",
            ),
            "the_image_small": self._create_simple_uploaded_file(
                orig_filename="image2k.png",
                dest_filename="the_image_small.png",
                content_type="image/png",
            ),
            "the_image_large": self._create_simple_uploaded_file(
                orig_filename="image15k.png",
                dest_filename="the_image_large.png",
                content_type="image/png",
            ),
        }
        form = TestImageModelForm(data={}, files=files)
        self.assertTrue(form.is_valid())

    def test_form_invalid_image_size(self):
        files = {
            "the_image": self._create_simple_uploaded_file(
                orig_filename="image2k.png",
                dest_filename="the_image.png",
                content_type="image/png",
            ),
            "the_image_small": self._create_simple_uploaded_file(
                orig_filename="image15k.png",
                dest_filename="the_image_small.png",
                content_type="image/png",
            ),
            "the_image_large": self._create_simple_uploaded_file(
                orig_filename="image2k.png",
                dest_filename="the_image_large.png",
                content_type="image/png",
            ),
        }
        form = TestImageModelForm(data={}, files=files)
        self.assertFalse(form.is_valid())
        self.assertEqual(len(form.errors), 2)
        self.assertEqual(len(form.errors["the_image_small"]), 3)
        self.assertEqual(
            form.errors["the_image_small"][0],
            u"File size exceeds limit: 14.2 KB. Limit is 10.0 KB.",
        )
        self.assertEqual(
            form.errors["the_image_small"][1],
            u"File height exceeds limit: 14586. Limit is 500.",
        )
        self.assertEqual(
            form.errors["the_image_small"][2],
            u"File width exceeds limit: 14586. Limit is 500.",
        )
        self.assertEqual(len(form.errors["the_image_large"]), 3)
        self.assertEqual(
            form.errors["the_image_large"][0],
            u"File size below limit: 2.1 KB. Limit is 10.0 KB.",
        )
        self.assertEqual(
            form.errors["the_image_large"][1],
            u"File height below limit: 2120. Limit is 3000.",
        )
        self.assertEqual(
            form.errors["the_image_large"][2],
            u"File width below limit: 2120. Limit is 3000.",
        )

    def test_form_invalid_filetype(self):
        form = self._create_bound_test_model_form(
            form_class=TestModelForm,
            orig_filename="document1k.pdf",
            dest_filename="the_file.pdf",
            content_type="application/pdf",
        )
        self.assertFalse(form.is_valid())
        self.assertEqual(len(form.errors), 1)
        self.assertEqual(len(form.errors["the_file"]), 1)
        self.assertEqual(
            form.errors["the_file"][0],
            u"Unsupported file type: application/pdf. "
            u"Allowed types are ['image/png'].",
        )

    def test_form_invalid_filetype_and_size(self):
        form = self._create_bound_test_model_form(
            form_class=TestModelForm,
            orig_filename="document15k.pdf",
            dest_filename="the_file.pdf",
            content_type="application/pdf",
        )
        self.assertFalse(form.is_valid())
        self.assertEqual(len(form.errors), 1)
        self.assertEqual(len(form.errors["the_file"]), 1)
        self.assertEqual(
            form.errors["the_file"][0],
            u"File size exceeds limit: 14.9 KB. Limit is 10.0 KB.",
        )

    def test_form_fake_filetype(self):
        form = self._create_bound_test_model_form(
            form_class=TestModelForm,
            orig_filename="document1k.pdf",
            dest_filename="the_file.pdf",
            content_type="image/png",
        )
        self.assertFalse(form.is_valid())
        self.assertEqual(len(form.errors), 1)
        self.assertEqual(len(form.errors["the_file"]), 1)
        self.assertEqual(
            form.errors["the_file"][0],
            u"Unsupported file type: application/pdf. "
            u"Allowed types are ['image/png'].",
        )

    def test_form_no_validate(self):
        form = self._create_bound_test_model_form(
            form_class=TestModelNoValidateForm,
            orig_filename="document15k.pdf",
            dest_filename="the_file.pdf",
            content_type="application/pdf",
        )
        self.assertTrue(form.is_valid())
        instance = form.save()

        self._check_file_url(instance.the_file, "the_file.pdf")

        instance.the_file.delete()
        instance.delete()

    def test_form_null_file(self):
        form = self._create_bound_test_model_form(form_class=TestModelNoValidateForm)
        self.assertTrue(form.is_valid())
        instance = form.save()

        self.assertEqual(instance.the_file, None)

        instance.delete()

    def test_nomodel_form_ok(self):
        form = self._create_bound_test_model_form(
            form_class=TestNoModelForm,
            orig_filename="image2k.png",
            dest_filename="the_file.png",
            content_type="image/png",
        )
        self.assertTrue(form.is_valid())

        for uploaded_file in form.files.values():
            uploaded_file.close()

    def test_nomodel_form_js_ok(self):
        form = self._create_bound_test_model_form(
            form_class=TestNoModelJsForm,
            orig_filename="image2k.png",
            dest_filename="the_file.png",
            content_type="image/png",
        )
        self.assertTrue(form.is_valid())

        for uploaded_file in form.files.values():
            uploaded_file.close()

    # -------
    # Utilities

    def _get_sample_file(self, filename):
        path = os.path.join(self.SAMPLE_FILES_PATH, filename)
        return open(path, mode="rb")

    def _check_file_url(self, filefield, filename):
        url = settings.MEDIA_URL + filefield.field.upload_to + "/" + filename
        self.assertEqual(filefield.url, url)

    def _create_simple_uploaded_file(self, orig_filename, dest_filename, content_type):
        return SimpleUploadedFile(
            name=dest_filename,
            content=self._get_sample_file(orig_filename).read(),
            content_type=content_type,
        )

    def _create_bound_test_model_form(
        self, form_class, orig_filename=None, dest_filename=None, content_type=None
    ):
        if orig_filename and dest_filename and content_type:
            uploaded_file = self._create_simple_uploaded_file(
                orig_filename, dest_filename, content_type=content_type,
            )
            files = {"the_file": uploaded_file}
        else:
            files = {}
        form = form_class(data={}, files=files)
        return form

    def _create_container(self, name):
        return TestContainer.objects.create(name=name)

    def _add_element(self, container, orig_filename, dest_filename):
        return container.test_elements.create(
            the_file=File(self._get_sample_file(orig_filename), dest_filename)
        )

    def _create_unbound_test_element_form(self, container):
        return TestElementForm(container=container)

    def _create_bound_test_element_form(
        self, container, orig_filename=None, dest_filename=None, content_type=None
    ):
        if orig_filename and dest_filename and content_type:
            uploaded_file = self._create_simple_uploaded_file(
                orig_filename, dest_filename, content_type=content_type,
            )
            files = {"the_file": uploaded_file}
        else:
            files = {}
        form = TestElementForm(container=container, data={}, files=files)
        return form



import os

BASE_DIR = os.path.dirname(os.path.realpath(__file__))

DEBUG = True

AUTHNET_LOGIN_ID = ""
AUTHNET_TRANSACTION_KEY = ""

SECRET_KEY = "m+qa*7_8t-=17zt_)9gi)4g%6w*v$xxkh6rwrys*bn9su+5%du"

INSTALLED_APPS = [
    "django.contrib.admin",
    "django.contrib.admindocs",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.messages",
    "django.contrib.sessions",
    "django.contrib.sites",
    "django.contrib.staticfiles",
    "constrainedfilefield",
    "constrainedfilefield.tests",
]

DATABASES = {
    "default": {"ENGINE": "django.db.backends.sqlite3"},
}

MIDDLEWARE = [
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.common.CommonMiddleware",
]

LANGUAGE_CODE = "en"

ROOT_URLCONF = "constrainedfilefield.tests.urls"

# MEDIA CONFIGURATION
# ------------------------------------------------------------------------------
# See: https://docs.djangoproject.com/en/stable/ref/settings/#media-root
MEDIA_ROOT = os.path.join(BASE_DIR, "media")

# See: https://docs.djangoproject.com/en/stable/ref/settings/#media-url
MEDIA_URL = "/media/"

# STATIC FILE CONFIGURATION
# ------------------------------------------------------------------------------
# See: https://docs.djangoproject.com/en/dev/ref/settings/#static-root
STATIC_ROOT = os.path.join(BASE_DIR, "staticfiles")

# See: https://docs.djangoproject.com/en/dev/ref/settings/#static-url
STATIC_URL = "/static/"

# See: https://docs.djangoproject.com/en/dev/ref/contrib/staticfiles/#staticfiles-finders
STATICFILES_FINDERS = [
    "django.contrib.staticfiles.finders.AppDirectoriesFinder",
]

LOGIN_URL = "/accounts/login/"

MANAGERS = []

SITE_ID = 1

USE_I18N = True

# TEMPLATE_DIRS = [os.path.join(os.path.dirname(__file__), 'templates')]
TEMPLATES = [
    {
        # See: https://docs.djangoproject.com/en/dev/ref/settings/#std:setting-TEMPLATES-BACKEND
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        # See: https://docs.djangoproject.com/en/dev/ref/settings/#template-dirs
        "DIRS": [os.path.join(os.path.dirname(__file__), "templates"),],
        "OPTIONS": {
            # See: https://docs.djangoproject.com/en/dev/ref/settings/#template-debug
            "debug": DEBUG,
            # See: https://docs.djangoproject.com/en/dev/ref/settings/#template-loaders
            # https://docs.djangoproject.com/en/dev/ref/templates/api/#loader-types
            "loaders": [
                "django.template.loaders.filesystem.Loader",
                "django.template.loaders.app_directories.Loader",
            ],
            # See: https://docs.djangoproject.com/en/dev/ref/settings/#template-context-processors
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.template.context_processors.i18n",
                "django.template.context_processors.media",
                "django.template.context_processors.static",
                "django.template.context_processors.tz",
                "django.contrib.messages.context_processors.messages",
                # Your stuff: custom template context processors go here
            ],
        },
    },
]



from django.shortcuts import render

from constrainedfilefield.tests import forms


def nomodel_form(request):
    """

    Parameters
    ----------
    request: HTTP request

    Returns
    --------
    HttpResponse
        with a context dictionary
    """
    context = {}
    if request.method == "POST":
        form = forms.TestNoModelForm(request.POST,)
        if form.is_valid():
            pass
        else:
            pass
    else:
        context["form"] = forms.TestNoModelForm()
    return render(request, "tests/form.html", context)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
from setuptools import find_packages, setup

from constrainedfilefield import __version__

REPO_URL = "https://github.com/mbourqui/django-constrainedfilefield/"

README = ""
for ext in ["md", "rst"]:
    try:
        with open(os.path.join(os.path.dirname(__file__), "README." + ext)) as readme:
            README = readme.read()
    except FileNotFoundError as fnfe:
        pass

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

FILETYPE_REQUIRE = [
    'python-magic-bin; platform_system=="Windows"',
    'python-magic >= 0.4.2; platform_system!="Windows"',
]

IMAGE_REQUIRE = ["Pillow >= 4.0.0"]

TESTS_REQUIRE = FILETYPE_REQUIRE + IMAGE_REQUIRE

setup(
    name="django-constrainedfilefield",
    version=__version__,
    author="Marc Bourqui",
    author_email="pypi.kemar@bourqui.org",
    license="BSD",
    description="This Django app adds a new field type, ConstrainedFileField, that has the "
    "capability of checking the document size and type.",
    long_description=README,
    url=REPO_URL,
    download_url=REPO_URL + "releases/tag/v" + __version__,
    packages=find_packages(),
    include_package_data=True,
    package_data={"": ["*.po", "*.mo"],},
    install_requires=['django>=1.11; python_version>="3.5"',],
    tests_require=TESTS_REQUIRE,
    extras_require={
        "filetype": FILETYPE_REQUIRE,
        "image": IMAGE_REQUIRE,
        "coverage": TESTS_REQUIRE,  # Used for travis
    },
    keywords="django filefield validation file",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Framework :: Django :: 1.11",
        "Framework :: Django :: 2.1",
        "Framework :: Django :: 2.2",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
        "Topic :: Utilities",
    ],
)



# -*- coding: utf-8 -*-
__all__ = ["ConstrainedFileField"]

import os

from django import forms
from django.conf import settings
from django.db import models
from django.template.defaultfilters import filesizeformat
from django.utils.translation import ugettext_lazy as _


class ConstrainedFileField(models.FileField):
    """
    A FielField with additional constraints. Namely, the file size and type can be restricted. If
    using the types, the magic library is required. Setting neither a file size nor type behaves
    like a regular FileField.

    Parameters
    ----------
    content_types : list of str
        List containing allowed content_types. Example: ['application/pdf', 'image/jpeg']
    max_upload_size : int
        Maximum file size allowed for upload, in bytes
            1 MB - 1048576 B - 1024**2 B - 2**20 B
            2.5 MB - 2621440 B
            5 MB - 5242880 B
            10 MB - 10485760 B
            20 MB - 20971520 B
            33 MiB - 2**25 B
            50 MB - 5242880 B
            100 MB 104857600 B
            250 MB - 214958080 B
            500 MB - 429916160 B
            1 GiB - 1024 MiB - 2**30 B
    js_checker : bool
        Add a javascript file size checker to the form field
    mime_lookup_length : int

    See Also
    --------
    Based on https://github.com/kaleidos/django-validated-file/blob/master/validatedfile/fields.py
    With inspiration from http://stackoverflow.com/a/9016664

    """

    description = _("A file field with constraints on size and/or type")

    def __init__(self, *args, **kwargs):
        self.max_upload_size = kwargs.pop("max_upload_size", 0)
        assert isinstance(self.max_upload_size, int) and self.max_upload_size >= 0
        self.content_types = kwargs.pop("content_types", [])
        self.mime_lookup_length = kwargs.pop("mime_lookup_length", 4096)
        assert isinstance(self.mime_lookup_length, int) and self.mime_lookup_length >= 0
        self.js_checker = kwargs.pop("js_checker", False)

        super(ConstrainedFileField, self).__init__(*args, **kwargs)

    def clean(self, *args, **kwargs):
        data = super(ConstrainedFileField, self).clean(*args, **kwargs)

        if self.max_upload_size and data.size > self.max_upload_size:
            # Ensure no one bypasses the js checker
            raise forms.ValidationError(
                _("File size exceeds limit: %(current_size)s. Limit is %(max_size)s.")
                % {
                    "max_size": filesizeformat(self.max_upload_size),
                    "current_size": filesizeformat(data.size),
                }
            )

        if self.content_types:
            import magic

            file = data.file
            uploaded_content_type = getattr(file, "content_type", "")

            # magic_file_path used only for Windows.
            magic_file_path = getattr(settings, "MAGIC_FILE_PATH", None)
            if magic_file_path and os.name == "nt":
                mg = magic.Magic(mime=True, magic_file=magic_file_path)
            else:
                mg = magic.Magic(mime=True)
            content_type_magic = mg.from_buffer(file.read(self.mime_lookup_length))
            file.seek(0)

            # Prefer mime-type from magic over mime-type from http header
            if uploaded_content_type != content_type_magic:
                uploaded_content_type = content_type_magic

            if uploaded_content_type not in self.content_types:
                raise forms.ValidationError(
                    _("Unsupported file type: %(type)s. Allowed types are %(allowed)s.")
                    % {"type": content_type_magic, "allowed": self.content_types}
                )
        return data

    def formfield(self, **kwargs):
        """
        Usual Form for a django.models.FileField with optional javascript file size
        checker. Can thus be customized as any other Form for a django.models.FileField.

        Parameters
        ----------
        kwargs
            will be passed to super().formfield()

        Returns
        -------
        django.forms.FileField

        """
        formfield = super(ConstrainedFileField, self).formfield(**kwargs)
        if self.js_checker:
            formfield.widget.attrs.update(
                {"onchange": "validateFileSize(this, 0, %d);" % (self.max_upload_size,)}
            )
        return formfield

    def deconstruct(self):
        name, path, args, kwargs = super(ConstrainedFileField, self).deconstruct()
        if self.max_upload_size:
            kwargs["max_upload_size"] = self.max_upload_size
        if self.content_types:
            kwargs["content_types"] = self.content_types
        if self.mime_lookup_length:
            kwargs["mime_lookup_length"] = self.mime_lookup_length
        if self.js_checker:
            kwargs["js_checker"] = self.js_checker
        return name, path, args, kwargs

    def __str__(self):
        if hasattr(self, "model"):
            return super(ConstrainedFileField).__str__()
        else:
            return self.__class__.__name__



import os.path

from django.conf import settings
from django.contrib.staticfiles.testing import StaticLiveServerTestCase
from django.test import Client


class ConstrainedFileFieldTest(StaticLiveServerTestCase):
    SAMPLE_FILES_PATH = os.path.join(settings.BASE_DIR, "sample_files")

    def test_nomodel_form_js_view_ok(self):
        c = Client()
        response = c.get("/nomodel/")
        # FIXME: validate response content: form is included as expected
        assert response.status_code == 200
        from django.contrib.staticfiles import finders

        self.assertIsNotNone(finders.find("constrainedfilefield/js/file_checker.js"))



from django import forms

from constrainedfilefield.tests.models import (
    TestModel,
    TestDocModel,
    TestImageModel,
    TestModelJs,
    TestModelNoValidate,
    TestElement,
)


class TestModelForm(forms.ModelForm):
    class Meta:
        model = TestModel
        fields = ["the_file"]


class TestDocModelForm(forms.ModelForm):
    class Meta:
        model = TestDocModel
        fields = ["the_file"]


class TestImageModelForm(forms.ModelForm):
    class Meta:
        model = TestImageModel
        fields = ["the_image", "the_image_small", "the_image_large"]


class TestModelFormJs(forms.ModelForm):
    class Meta:
        model = TestModelJs
        fields = ["the_file"]


class TestModelNoValidateForm(forms.ModelForm):
    class Meta:
        model = TestModelNoValidate
        fields = ["the_file"]


class TestElementForm(forms.ModelForm):
    the_file = forms.FileField(required=False,)

    class Meta:
        model = TestElement
        fields = ["the_file"]

    def __init__(self, container, *args, **kwargs):
        super(TestElementForm, self).__init__(*args, **kwargs)
        self.container = container
        self.fields["the_file"].validators[0].update_quota(
            items=self.container.test_elements.all(), attr_name="the_file",
        )

    def save(self, *args, **kwargs):
        element = super(TestElementForm, self).save(commit=False)
        element.container = self.container
        element.save()


class TestNoModelForm(forms.Form):
    from constrainedfilefield.fields import ConstrainedFileField

    the_file = ConstrainedFileField(
        null=True,
        blank=True,
        upload_to="testfile",
        content_types=["image/png"],
        max_upload_size=10240,
    ).formfield()


class TestNoModelJsForm(forms.Form):
    from constrainedfilefield.fields import ConstrainedFileField

    the_file = ConstrainedFileField(
        null=True,
        blank=True,
        upload_to="testfile",
        content_types=["image/png"],
        max_upload_size=10240,
        js_checker=True,
    ).formfield()
